@@ -10,6 +10,7 @@
 #include "buildrun.h"
 #include "session.h"
 #include "util.h"
+#include "hash.h"
 
 #include <cstdlib>
 #include <fstream>
@@ -30,8 +31,6 @@ extern "C" {
 
 using namespace std;
 
-static int uprobes_pass (systemtap_session& s);
-
 /* Adjust and run make_cmd to build a kernel module. */
 static int
 run_make_cmd(systemtap_session& s, vector<string>& make_cmd,
@@ -89,6 +88,27 @@ run_make_cmd(systemtap_session& s, vector<string>& make_cmd,
   return rc;
 }
 
+static vector<string>
+make_make_cmd(systemtap_session& s, const string& dir)
+{
+  vector<string> make_cmd;
+  make_cmd.push_back("make");
+  make_cmd.push_back("-C");
+  make_cmd.push_back(s.kernel_build_tree);
+  make_cmd.push_back("M=" + dir); // need make-quoting?
+  make_cmd.push_back("modules");
+
+  // Add architecture, except for old powerpc (RHBZ669082)
+  if (s.architecture != "powerpc" ||
+      (strverscmp (s.kernel_base_release.c_str(), "2.6.15") >= 0))
+    make_cmd.push_back("ARCH=" + s.architecture); // need make-quoting?
+
+  // Add any custom kbuild flags
+  make_cmd.insert(make_cmd.end(), s.kbuildflags.begin(), s.kbuildflags.end());
+
+  return make_cmd;
+}
+
 static void
 output_autoconf(systemtap_session& s, ofstream& o, const char *autoconf_c,
                 const char *deftrue, const char *deffalse)
@@ -215,6 +235,8 @@ compile_pass (systemtap_session& s)
   output_autoconf(s, o, "autoconf-ring_buffer-flags.c", "STAPCONF_RING_BUFFER_FLAGS", NULL);
   output_autoconf(s, o, "autoconf-kallsyms-on-each-symbol.c", "STAPCONF_KALLSYMS_ON_EACH_SYMBOL", NULL);
   output_autoconf(s, o, "autoconf-walk-stack.c", "STAPCONF_WALK_STACK", NULL);
+  output_autoconf(s, o, "autoconf-stacktrace_ops-warning.c",
+                  "STAPCONF_STACKTRACE_OPS_WARNING", NULL);
   output_autoconf(s, o, "autoconf-mm-context-vdso.c", "STAPCONF_MM_CONTEXT_VDSO", NULL);
   output_autoconf(s, o, "autoconf-blk-types.c", "STAPCONF_BLK_TYPES", NULL);
   output_autoconf(s, o, "autoconf-perf-structpid.c", "STAPCONF_PERF_STRUCTPID", NULL);
@@ -278,22 +300,7 @@ compile_pass (systemtap_session& s)
     }
 
   // Run make
-  vector<string> make_cmd;
-  make_cmd.push_back("make");
-  make_cmd.push_back("-C");
-  make_cmd.push_back(module_dir);
-  make_cmd.push_back("M=" + s.tmpdir); // need make-quoting?
-
-  // Add architecture, except for old powerpc (RHBZ669082)
-  if (s.architecture != "powerpc" ||
-      (strverscmp (s.kernel_base_release.c_str(), "2.6.15") >= 0))
-    make_cmd.push_back("ARCH=" + s.architecture); // need make-quoting?
-
-  // Add any custom kbuild flags
-  make_cmd.insert(make_cmd.end(), s.kbuildflags.begin(), s.kbuildflags.end());
-
-  make_cmd.push_back("modules");
-
+  vector<string> make_cmd = make_make_cmd(s, s.tmpdir);
   rc = run_make_cmd(s, make_cmd);
   if (rc)
     s.set_try_server ();
@@ -302,95 +309,13 @@ compile_pass (systemtap_session& s)
 
 /*
  * If uprobes was built as part of the kernel build (either built-in
- * or as a module), the uprobes exports should show up in either
- * s.kernel_build_tree / Module.symvers.  Return true if so.
+ * or as a module), the uprobes exports should show up.  This is to be
+ * as distinct from the stap-built uprobes.ko from the runtime.
  */
 static bool
 kernel_built_uprobes (systemtap_session& s)
 {
-  vector<string> grep_cmd;
-  grep_cmd.push_back ("/bin/grep");
-  grep_cmd.push_back ("-q");
-  grep_cmd.push_back ("unregister_uprobe");
-  grep_cmd.push_back (s.kernel_build_tree + "/Module.symvers");
-
-  return (stap_system (s.verbose, grep_cmd) == 0);
-}
-
-/*
- * We only want root, the owner of the uprobes build directory
- * and members of the group owning the uprobes build directory
- * modifying uprobes.
- */
-static bool
-may_build_uprobes (const systemtap_session& s)
-{
-  // root may build uprobes.
-  uid_t euid = geteuid ();
-  if (euid == 0)
-    return true;
-
-  // Get information on the build directory.
-  string uprobes_home = s.runtime_path + "/uprobes";
-  struct stat file_info;
-  if (stat(uprobes_home.c_str(), &file_info) != 0) {
-    clog << _("Unable to obtain information on ") << uprobes_home << '.' << endl;
-    return false;
-  }
-
-  // The owner of the build directory may build uprobes.
-  if (euid == file_info.st_uid)
-    return true;
-
-  // Members of the group owner of the build directory may build uprobes.
-  if (in_group_id (file_info.st_gid))
-    return true;
-
-  return false;
-}
-
-/*
- * Use "make -q" with a fake target to
- * verify that uprobes doesn't need to be rebuilt.
- */
-static bool
-verify_uprobes_uptodate (systemtap_session& s)
-{
-  if (s.verbose > 1)
-    clog << _("Pass 4, preamble: verifying that SystemTap's version of uprobes is up to date.")
-	 << endl;
-
-  string uprobes_home = s.runtime_path + "/uprobes";
-  vector<string> make_cmd;
-  make_cmd.push_back("make");
-  make_cmd.push_back("-q");
-  make_cmd.push_back("-C");
-  make_cmd.push_back(uprobes_home);
-  make_cmd.push_back("uprobes.ko");
-  int rc = run_make_cmd(s, make_cmd);
-  if (rc) {
-    clog << _("SystemTap's version of uprobes is out of date.") << endl;
-
-    struct stat file_info;
-    if (stat(uprobes_home.c_str(), &file_info) != 0) {
-      clog << _("Unable to obtain information on ") << uprobes_home << '.' << endl;
-    }
-    else {
-      struct passwd *owner = getpwuid (file_info.st_uid);
-      string owner_name = owner == NULL ? _("The owner of ") + uprobes_home :
-	                                  owner->pw_name;
-      if (owner_name == "root")
-	owner_name = "";
-      struct group *owner_group = getgrgid (file_info.st_gid);
-      string owner_group_name = owner_group == NULL ? _("The owner group of ") + uprobes_home :
-	                                              owner_group->gr_name;
-      clog << _F("As root, %s%s or a member of the '%s' group, run\n",
-                 owner_name.c_str(), (owner_name.empty() ? "" : ", "), owner_group_name.c_str());
-      clog << "\"make -C " << uprobes_home << "\"." << endl;
-    }
-  }
-
-  return rc;
+  return (s.kernel_exports.find("unregister_uprobe") != s.kernel_exports.end());
 }
 
 static int
@@ -400,35 +325,93 @@ make_uprobes (systemtap_session& s)
     clog << _("Pass 4, preamble: (re)building SystemTap's version of uprobes.")
 	 << endl;
 
-  string uprobes_home = s.runtime_path + "/uprobes";
-  vector<string> make_cmd;
-  make_cmd.push_back("make");
-  make_cmd.push_back("-C");
-  make_cmd.push_back(uprobes_home);
-  int rc = run_make_cmd(s, make_cmd);
+  // create a subdirectory for the uprobes module
+  string dir(s.tmpdir + "/uprobes");
+  if (create_dir(dir.c_str()) != 0)
+    {
+      if (! s.suppress_warnings)
+        cerr << _("Warning: failed to create directory for build uprobes.") << endl;
+      s.set_try_server ();
+      return 1;
+    }
+
+  // create a simple Makefile
+  string makefile(dir + "/Makefile");
+  ofstream omf(makefile.c_str());
+  omf << "obj-m := uprobes.o" << endl;
+  // RHBZ 655231: later rhel6 kernels' module-signing kbuild logic breaks out-of-tree modules
+  omf << "CONFIG_MODULE_SIG := n" << endl;
+  omf.close();
+
+  // create a simple #include-chained source file
+  string runtimesourcefile(s.runtime_path + "/uprobes/uprobes.c");
+  string sourcefile(dir + "/uprobes.c");
+  ofstream osrc(sourcefile.c_str());
+  osrc << "#include \"" << runtimesourcefile << "\"" << endl;
+  osrc.close();
+
+  // make the module
+  vector<string> make_cmd = make_make_cmd(s, dir);
+  bool quiet = (s.verbose < 4);
+  int rc = run_make_cmd(s, make_cmd, quiet, quiet);
+  if (!rc && !copy_file(dir + "/Module.symvers",
+                        s.tmpdir + "/Module.symvers"))
+    rc = -1;
+
   if (s.verbose > 1)
     clog << _("uprobes rebuild exit code: ") << rc << endl;
   if (rc)
     s.set_try_server ();
+  else
+    s.uprobes_path = dir + "/uprobes.ko";
   return rc;
 }
 
-/*
- * Copy uprobes' exports (in Module.symvers) into the temporary directory
- * so the script-module build can find them.
- */
-static int
-copy_uprobes_symbols (const systemtap_session& s)
+static bool
+get_cached_uprobes(systemtap_session& s)
 {
-  vector<string> cp_cmd;
-  cp_cmd.push_back ("/bin/cp");
-  cp_cmd.push_back (s.runtime_path + "/uprobes/Module.symvers");
-  cp_cmd.push_back (s.tmpdir);
+  s.uprobes_hash = s.use_cache ? find_uprobes_hash(s) : "";
+  if (!s.uprobes_hash.empty())
+    {
+      // NB: We always put uprobes.ko in its own directory, especially so
+      // stap-serverd can more easily locate it.
+      string dir(s.tmpdir + "/uprobes");
+      if (create_dir(dir.c_str()) != 0)
+        return false;
 
-  return stap_system (s.verbose, cp_cmd);
+      string cacheko = s.uprobes_hash + ".ko";
+      string tmpko = dir + "/uprobes.ko";
+
+      // The symvers file still needs to go in the script module's directory.
+      string cachesyms = s.uprobes_hash + ".symvers";
+      string tmpsyms = s.tmpdir + "/Module.symvers";
+
+      if (get_file_size(cacheko) > 0 && copy_file(cacheko, tmpko) &&
+          get_file_size(cachesyms) > 0 && copy_file(cachesyms, tmpsyms))
+        {
+          s.uprobes_path = tmpko;
+          return true;
+        }
+    }
+  return false;
 }
 
-static int
+static void
+set_cached_uprobes(systemtap_session& s)
+{
+  if (s.use_cache && !s.uprobes_hash.empty())
+    {
+      string cacheko = s.uprobes_hash + ".ko";
+      string tmpko = s.tmpdir + "/uprobes/uprobes.ko";
+      copy_file(tmpko, cacheko);
+
+      string cachesyms = s.uprobes_hash + ".symvers";
+      string tmpsyms = s.tmpdir + "/uprobes/Module.symvers";
+      copy_file(tmpsyms, cachesyms);
+    }
+}
+
+int
 uprobes_pass (systemtap_session& s)
 {
   if (!s.need_uprobes || kernel_built_uprobes(s))
@@ -441,30 +424,25 @@ uprobes_pass (systemtap_session& s)
   }
 
   /*
-   * We need to use the version of uprobes that comes with SystemTap, so
-   * we may need to rebuild uprobes.ko there.  Unfortunately, this is
-   * never a no-op; e.g., the modpost step gets run every time.  Only
-   * certain users can build uprobes, so we keep the uprobes directory
-   * writable only by those users.  But that means that other users
-   * can't run the make even if everything's up to date.
-   *
-   * So for the other users, we just verify that uprobes doesn't need
-   * to be rebuilt.  If that's not so, stap must fail.
+   * We need to use the version of uprobes that comes with SystemTap.  Try to
+   * get it from the cache first.  If not found, build it and try to save it to
+   * the cache for future reuse.
    */
-  int rc;
-  if (may_build_uprobes (s))
-    rc = make_uprobes(s);
-  else
-    rc = verify_uprobes_uptodate(s);
-  if (rc == 0)
-    rc = copy_uprobes_symbols(s);
+  int rc = 0;
+  if (!get_cached_uprobes(s))
+    {
+      rc = make_uprobes(s);
+      if (!rc)
+        set_cached_uprobes(s);
+    }
   if (rc)
     s.set_try_server ();
   return rc;
 }
 
 vector<string>
-make_run_command (systemtap_session& s, const string& module)
+make_run_command (systemtap_session& s, const string& module,
+                  const string& version)
 {
   // for now, just spawn staprun
   vector<string> staprun_cmd;
@@ -510,6 +488,9 @@ make_run_command (systemtap_session& s, const string& module)
   if (s.load_only)
     staprun_cmd.push_back(s.output_file.empty() ? "-L" : "-D");
 
+  if(!s.modname_given && (strverscmp("1.6", version.c_str()) <= 0))
+    staprun_cmd.push_back("-R");
+
   if (!s.size_option.empty())
     {
       staprun_cmd.push_back("-S");
@@ -595,25 +576,18 @@ make_tracequery(systemtap_session& s, string& name,
   osrc.close();
 
   // make the module
-  vector<string> make_cmd;
-  make_cmd.push_back("make");
-  make_cmd.push_back("-C");
-  make_cmd.push_back(s.kernel_build_tree);
-  make_cmd.push_back("M=" + dir); // need make-quoting?
-  make_cmd.push_back("modules");
-
-  // Add architecture, except for old powerpc (RHBZ669082)
-  if (s.architecture != "powerpc" ||
-      (strverscmp (s.kernel_base_release.c_str(), "2.6.15") >= 0))
-    make_cmd.push_back("ARCH=" + s.architecture); // need make-quoting?
-
-  // Add any custom kbuild flags
-  make_cmd.insert(make_cmd.end(), s.kbuildflags.begin(), s.kbuildflags.end());
-
+  vector<string> make_cmd = make_make_cmd(s, dir);
   bool quiet = (s.verbose < 4);
   int rc = run_make_cmd(s, make_cmd, quiet, quiet);
   if (rc)
     s.set_try_server ();
+
+  // XXX: sometimes we fail a tracequery due to PR9993 / PR11649 type
+  // kernel trace header problems.  In this case, due to PR12729,
+  // we get a lovely "Warning: make exited with status: 2" but no
+  // other useful diagnostic.  -vvvv would let a user see what's up,
+  // but the user can't fix the problem even with that.
+
   return rc;
 }
 
@@ -666,21 +640,7 @@ make_typequery_kmod(systemtap_session& s, const vector<string>& headers, string&
   osrc.close();
 
   // make the module
-  vector<string> make_cmd;
-  make_cmd.push_back("make");
-  make_cmd.push_back("-C");
-  make_cmd.push_back(s.kernel_build_tree);
-  make_cmd.push_back("M=" + dir); // need make-quoting?
-  make_cmd.push_back("modules");
-
-  // Add architecture, except for old powerpc (RHBZ669082)
-  if (s.architecture != "powerpc" ||
-      (strverscmp (s.kernel_base_release.c_str(), "2.6.15") >= 0))
-    make_cmd.push_back("ARCH=" + s.architecture); // need make-quoting?
-
-  // Add any custom kbuild flags
-  make_cmd.insert(make_cmd.end(), s.kbuildflags.begin(), s.kbuildflags.end());
-
+  vector<string> make_cmd = make_make_cmd(s, dir);
   bool quiet = (s.verbose < 4);
   int rc = run_make_cmd(s, make_cmd, quiet, quiet);
   if (rc)
