@@ -12,8 +12,11 @@
 #include "elaborate.h"
 
 int compile_pass (systemtap_session& s);
+int uprobes_pass (systemtap_session& s);
+
 std::vector<std::string> make_run_command (systemtap_session& s,
-                                           const std::string& module="");
+                                           const std::string& module="",
+                                           const std::string& version=VERSION);
 
 int make_tracequery(systemtap_session& s, std::string& name,
                     const std::vector<std::string>& headers);
