@@ -26,6 +26,14 @@
 #define JOIN5x(a,b,c,d,e,f) a##_##b##c##d##e##f
 #define JOIN6(a,b,c,d,e,f,g) JOIN6x(a,b,c,d,e,f,g)
 #define JOIN6x(a,b,c,d,e,f,g) a##_##b##c##d##e##f##g
+#define JOIN7(a,b,c,d,e,f,g,h) JOIN7x(a,b,c,d,e,f,g,h)
+#define JOIN7x(a,b,c,d,e,f,g,h) a##_##b##c##d##e##f##g##h
+#define JOIN8(a,b,c,d,e,f,g,h,i) JOIN8x(a,b,c,d,e,f,g,h,i)
+#define JOIN8x(a,b,c,d,e,f,g,h,i) a##_##b##c##d##e##f##g##h##i
+#define JOIN9(a,b,c,d,e,f,g,h,i,j) JOIN9x(a,b,c,d,e,f,g,h,i,j)
+#define JOIN9x(a,b,c,d,e,f,g,h,i,j) a##_##b##c##d##e##f##g##h##i##j
+#define JOIN10(a,b,c,d,e,f,g,h,i,j,k) JOIN10x(a,b,c,d,e,f,g,h,i,j,k)
+#define JOIN10x(a,b,c,d,e,f,g,h,i,j,k) a##_##b##c##d##e##f##g##h##i##j##k
 
 #include "map.h"
 
@@ -162,6 +170,113 @@
 #define KEY5_HASH JOIN(KEY5NAME,hash)
 #endif /* defined(KEY5_TYPE) */
 
+#if defined (KEY6_TYPE)
+#undef KEY_ARITY
+#define KEY_ARITY 6
+#if KEY6_TYPE == STRING
+#define KEY6TYPE char*
+#define KEY6NAME str
+#define KEY6N s
+#define KEY6STOR char key6[MAP_STRING_LENGTH]
+#define KEY6CPY(m) str_copy(m->key6, key6)
+#else
+#define KEY6TYPE int64_t
+#define KEY6NAME int64
+#define KEY6N i
+#define KEY6STOR int64_t key6
+#define KEY6CPY(m) m->key6=key6
+#endif
+#define KEY6_EQ_P JOIN(KEY6NAME,eq_p)
+#define KEY6_HASH JOIN(KEY6NAME,hash)
+#endif /* defined(KEY6_TYPE) */
+
+#if defined (KEY7_TYPE)
+#undef KEY_ARITY
+#define KEY_ARITY 7
+#if KEY7_TYPE == STRING
+#define KEY7TYPE char*
+#define KEY7NAME str
+#define KEY7N s
+#define KEY7STOR char key7[MAP_STRING_LENGTH]
+#define KEY7CPY(m) str_copy(m->key7, key7)
+#else
+#define KEY7TYPE int64_t
+#define KEY7NAME int64
+#define KEY7N i
+#define KEY7STOR int64_t key7
+#define KEY7CPY(m) m->key7=key7
+#endif
+#define KEY7_EQ_P JOIN(KEY7NAME,eq_p)
+#define KEY7_HASH JOIN(KEY7NAME,hash)
+#endif /* defined(KEY7_TYPE) */
+
+#if defined (KEY7_TYPE)
+#undef KEY_ARITY
+#define KEY_ARITY 7
+#if KEY7_TYPE == STRING
+#define KEY7TYPE char*
+#define KEY7NAME str
+#define KEY7N s
+#define KEY7STOR char key7[MAP_STRING_LENGTH]
+#define KEY7CPY(m) str_copy(m->key7, key7)
+#else
+#define KEY7TYPE int64_t
+#define KEY7NAME int64
+#define KEY7N i
+#define KEY7STOR int64_t key7
+#define KEY7CPY(m) m->key7=key7
+#endif
+#define KEY7_EQ_P JOIN(KEY7NAME,eq_p)
+#define KEY7_HASH JOIN(KEY7NAME,hash)
+#endif /* defined(KEY7_TYPE) */
+
+#if defined (KEY8_TYPE)
+#undef KEY_ARITY
+#define KEY_ARITY 8
+#if KEY8_TYPE == STRING
+#define KEY8TYPE char*
+#define KEY8NAME str
+#define KEY8N s
+#define KEY8STOR char key8[MAP_STRING_LENGTH]
+#define KEY8CPY(m) str_copy(m->key8, key8)
+#else
+#define KEY8TYPE int64_t
+#define KEY8NAME int64
+#define KEY8N i
+#define KEY8STOR int64_t key8
+#define KEY8CPY(m) m->key8=key8
+#endif
+#define KEY8_EQ_P JOIN(KEY8NAME,eq_p)
+#define KEY8_HASH JOIN(KEY8NAME,hash)
+#endif /* defined(KEY8_TYPE) */
+
+#if defined (KEY9_TYPE)
+#undef KEY_ARITY
+#define KEY_ARITY 9
+#if KEY9_TYPE == STRING
+#define KEY9TYPE char*
+#define KEY9NAME str
+#define KEY9N s
+#define KEY9STOR char key9[MAP_STRING_LENGTH]
+#define KEY9CPY(m) str_copy(m->key9, key9)
+#else
+#define KEY9TYPE int64_t
+#define KEY9NAME int64
+#define KEY9N i
+#define KEY9STOR int64_t key9
+#define KEY9CPY(m) m->key9=key9
+#endif
+#define KEY9_EQ_P JOIN(KEY9NAME,eq_p)
+#define KEY9_HASH JOIN(KEY9NAME,hash)
+#endif /* defined(KEY9_TYPE) */
+
+/* Not so many, cowboy! */
+#if defined (KEY10_TYPE)
+#error "excessive key arity == too many array indexes"
+#endif
+
+
+
 #if KEY_ARITY == 1
 #define KEYSYM(x) JOIN2(x,KEY1N,VALN)
 #define ALLKEYS(x) x##1
@@ -187,6 +302,26 @@
 #define ALLKEYS(x) x##1, x##2, x##3, x##4, x##5
 #define ALLKEYSD(x) KEY1TYPE x##1, KEY2TYPE x##2, KEY3TYPE x##3, KEY4TYPE x##4, KEY5TYPE x##5
 #define KEYCPY(m) {KEY1CPY(m);KEY2CPY(m);KEY3CPY(m);KEY4CPY(m);KEY5CPY(m);}
+#elif KEY_ARITY == 6
+#define KEYSYM(x) JOIN7(x,KEY1N,KEY2N,KEY3N,KEY4N,KEY5N,KEY6N,VALN)
+#define ALLKEYS(x) x##1, x##2, x##3, x##4, x##5, x##6
+#define ALLKEYSD(x) KEY1TYPE x##1, KEY2TYPE x##2, KEY3TYPE x##3, KEY4TYPE x##4, KEY5TYPE x##5, KEY6TYPE x##6
+#define KEYCPY(m) {KEY1CPY(m);KEY2CPY(m);KEY3CPY(m);KEY4CPY(m);KEY5CPY(m);KEY6CPY(m);}
+#elif KEY_ARITY == 7
+#define KEYSYM(x) JOIN8(x,KEY1N,KEY2N,KEY3N,KEY4N,KEY5N,KEY6N,KEY7N,VALN)
+#define ALLKEYS(x) x##1, x##2, x##3, x##4, x##5, x##6, x##7
+#define ALLKEYSD(x) KEY1TYPE x##1, KEY2TYPE x##2, KEY3TYPE x##3, KEY4TYPE x##4, KEY5TYPE x##5, KEY6TYPE x##6, KEY7TYPE x##7
+#define KEYCPY(m) {KEY1CPY(m);KEY2CPY(m);KEY3CPY(m);KEY4CPY(m);KEY5CPY(m);KEY6CPY(m);KEY7CPY(m);}
+#elif KEY_ARITY == 8
+#define KEYSYM(x) JOIN9(x,KEY1N,KEY2N,KEY3N,KEY4N,KEY5N,KEY6N,KEY7N,KEY8N,VALN)
+#define ALLKEYS(x) x##1, x##2, x##3, x##4, x##5, x##6, x##7, x##8
+#define ALLKEYSD(x) KEY1TYPE x##1, KEY2TYPE x##2, KEY3TYPE x##3, KEY4TYPE x##4, KEY5TYPE x##5, KEY6TYPE x##6, KEY7TYPE x##7, KEY8TYPE x##8
+#define KEYCPY(m) {KEY1CPY(m);KEY2CPY(m);KEY3CPY(m);KEY4CPY(m);KEY5CPY(m);KEY6CPY(m);KEY7CPY(m);KEY8CPY(m);}
+#elif KEY_ARITY == 9
+#define KEYSYM(x) JOIN10(x,KEY1N,KEY2N,KEY3N,KEY4N,KEY5N,KEY6N,KEY7N,KEY8N,KEY9N,VALN)
+#define ALLKEYS(x) x##1, x##2, x##3, x##4, x##5, x##6, x##7, x##8, x##9
+#define ALLKEYSD(x) KEY1TYPE x##1, KEY2TYPE x##2, KEY3TYPE x##3, KEY4TYPE x##4, KEY5TYPE x##5, KEY6TYPE x##6, KEY7TYPE x##7, KEY8TYPE x##8, KEY9TYPE x##9
+#define KEYCPY(m) {KEY1CPY(m);KEY2CPY(m);KEY3CPY(m);KEY4CPY(m);KEY5CPY(m);KEY6CPY(m);KEY7CPY(m);KEY8CPY(m);KEY9CPY(m);}
 #endif
 
 /* */
@@ -208,6 +343,18 @@ struct KEYSYM(map_node) {
 	KEY4STOR;
 #if KEY_ARITY > 4
 	KEY5STOR;
+#if KEY_ARITY > 5
+	KEY6STOR;
+#if KEY_ARITY > 6
+	KEY7STOR;
+#if KEY_ARITY > 7
+	KEY8STOR;
+#if KEY_ARITY > 8
+	KEY9STOR;
+#endif
+#endif
+#endif
+#endif
 #endif
 #endif
 #endif
@@ -229,7 +376,6 @@ static key_data KEYSYM(map_get_key) (struct map_node *mn, int n, int *type)
 	key_data ptr;
 	struct KEYSYM(map_node) *m = (struct KEYSYM(map_node) *)mn;	
 
-	dbug ("n = %d type=%lx\n", n, type);
 	if (n > KEY_ARITY || n < 1) {
 		if (type)
 			*type = END;
@@ -267,6 +413,34 @@ static key_data KEYSYM(map_get_key) (struct map_node *mn, int n, int *type)
 		if (type)
 			*type = type_to_enum(KEY5TYPE);
 		break;
+#if KEY_ARITY > 5
+	case 6:
+		ptr = (key_data)m->key6;
+		if (type)
+			*type = type_to_enum(KEY6TYPE);
+		break;
+#if KEY_ARITY > 6
+	case 7:
+		ptr = (key_data)m->key7;
+		if (type)
+			*type = type_to_enum(KEY7TYPE);
+		break;
+#if KEY_ARITY > 7
+	case 8:
+		ptr = (key_data)m->key8;
+		if (type)
+			*type = type_to_enum(KEY8TYPE);
+		break;
+#if KEY_ARITY > 8
+	case 9:
+		ptr = (key_data)m->key9;
+		if (type)
+			*type = type_to_enum(KEY9TYPE);
+		break;
+#endif
+#endif
+#endif
+#endif
 #endif
 #endif
 #endif
@@ -310,6 +484,34 @@ static unsigned int KEYSYM(keycheck) (ALLKEYSD(key))
 	if (key5 == NULL)
 		return 0;
 #endif
+
+#if KEY_ARITY > 5
+#if KEY6_TYPE == STRING
+	if (key6 == NULL)
+		return 0;
+#endif
+
+#if KEY_ARITY > 6
+#if KEY7_TYPE == STRING
+	if (key7 == NULL)
+		return 0;
+#endif
+
+#if KEY_ARITY > 7
+#if KEY8_TYPE == STRING
+	if (key8 == NULL)
+		return 0;
+#endif
+
+#if KEY_ARITY > 8
+#if KEY9_TYPE == STRING
+	if (key9 == NULL)
+		return 0;
+#endif
+#endif
+#endif
+#endif
+#endif
 #endif
 #endif
 #endif
@@ -328,6 +530,18 @@ static unsigned int KEYSYM(hash) (ALLKEYSD(key))
 	hash ^= KEY4_HASH(key4);
 #if KEY_ARITY > 4
 	hash ^= KEY5_HASH(key5);
+#if KEY_ARITY > 5
+	hash ^= KEY6_HASH(key6);
+#if KEY_ARITY > 6
+	hash ^= KEY7_HASH(key7);
+#if KEY_ARITY > 7
+	hash ^= KEY8_HASH(key8);
+#if KEY_ARITY > 8
+	hash ^= KEY9_HASH(key9);
+#endif
+#endif
+#endif
+#endif
 #endif
 #endif
 #endif
@@ -337,7 +551,7 @@ static unsigned int KEYSYM(hash) (ALLKEYSD(key))
 
 
 #if VALUE_TYPE == INT64 || VALUE_TYPE == STRING
-MAP KEYSYM(_stp_map_new) (unsigned max_entries)
+static MAP KEYSYM(_stp_map_new) (unsigned max_entries)
 {
 	MAP m = _stp_map_new (max_entries, VALUE_TYPE, sizeof(struct KEYSYM(map_node)), 0);
 	if (m)
@@ -348,7 +562,7 @@ MAP KEYSYM(_stp_map_new) (unsigned max_entries)
 /* _stp_map_new_key1_key2...val (num, HIST_LINEAR, start, end, interval) */
 /* _stp_map_new_key1_key2...val (num, HIST_LOG) */ 
 
-MAP KEYSYM(_stp_map_new) (unsigned max_entries, int htype, ...)
+static MAP KEYSYM(_stp_map_new) (unsigned max_entries, int htype, ...)
 {
 	int start=0, stop=0, interval=0;
 	MAP m;
@@ -359,7 +573,6 @@ MAP KEYSYM(_stp_map_new) (unsigned max_entries, int htype, ...)
 		start = va_arg(ap, int);
 		stop = va_arg(ap, int);
 		interval = va_arg(ap, int);
-		// dbug ("start=%d stop=%d interval=%d\n", start, stop, interval);
 		va_end (ap);
 	}
 
@@ -386,7 +599,7 @@ MAP KEYSYM(_stp_map_new) (unsigned max_entries, int htype, ...)
 }
 
 #endif /* VALUE_TYPE */
-int KEYSYM(__stp_map_set) (MAP map, ALLKEYSD(key), VSTYPE val, int add)
+static int KEYSYM(__stp_map_set) (MAP map, ALLKEYSD(key), VSTYPE val, int add)
 {
 	unsigned int hv;
 	struct hlist_head *head;
@@ -404,7 +617,6 @@ int KEYSYM(__stp_map_set) (MAP map, ALLKEYSD(key), VSTYPE val, int add)
 
 	hlist_for_each(e, head) {
 		n = (struct KEYSYM(map_node) *)((long)e - sizeof(struct list_head));
-		//dbug ("n=%lx  key1=%ld n->key1=%ld\n", (long)n, key1, n->key1);
 		if (KEY1_EQ_P(n->key1, key1)
 #if KEY_ARITY > 1
 		    && KEY2_EQ_P(n->key2, key2)
@@ -414,6 +626,18 @@ int KEYSYM(__stp_map_set) (MAP map, ALLKEYSD(key), VSTYPE val, int add)
 		    && KEY4_EQ_P(n->key4, key4)
 #if KEY_ARITY > 4
 		    && KEY5_EQ_P(n->key5, key5)
+#if KEY_ARITY > 5
+		    && KEY6_EQ_P(n->key6, key6)
+#if KEY_ARITY > 6
+		    && KEY7_EQ_P(n->key7, key7)
+#if KEY_ARITY > 7
+		    && KEY8_EQ_P(n->key8, key8)
+#if KEY_ARITY > 8
+		    && KEY9_EQ_P(n->key9, key9)
+#endif
+#endif
+#endif
+#endif
 #endif
 #endif
 #endif
@@ -423,8 +647,6 @@ int KEYSYM(__stp_map_set) (MAP map, ALLKEYSD(key), VSTYPE val, int add)
 		}
 	}
 	/* key not found */
-	dbug("key not found\n");
-
 	n = (struct KEYSYM(map_node)*)_new_map_create (map, head);
 	if (n == NULL)
 		return -1;
@@ -432,18 +654,18 @@ int KEYSYM(__stp_map_set) (MAP map, ALLKEYSD(key), VSTYPE val, int add)
 	return MAP_SET_VAL(map,(struct map_node *)n, val, 0);
 }
 
-int KEYSYM(_stp_map_set) (MAP map, ALLKEYSD(key), VSTYPE val)
+static int KEYSYM(_stp_map_set) (MAP map, ALLKEYSD(key), VSTYPE val)
 {
 	return KEYSYM(__stp_map_set) (map, ALLKEYS(key), val, 0);
 }
 
-int KEYSYM(_stp_map_add) (MAP map, ALLKEYSD(key), VSTYPE val)
+static int KEYSYM(_stp_map_add) (MAP map, ALLKEYSD(key), VSTYPE val)
 {
 	return KEYSYM(__stp_map_set) (map, ALLKEYS(key), val, 1);
 }
 
 
-VALTYPE KEYSYM(_stp_map_get) (MAP map, ALLKEYSD(key))
+static VALTYPE KEYSYM(_stp_map_get) (MAP map, ALLKEYSD(key))
 {
 	unsigned int hv;
 	struct hlist_head *head;
@@ -467,6 +689,18 @@ VALTYPE KEYSYM(_stp_map_get) (MAP map, ALLKEYSD(key))
 		    && KEY4_EQ_P(n->key4, key4)
 #if KEY_ARITY > 4
 		    && KEY5_EQ_P(n->key5, key5)
+#if KEY_ARITY > 5
+		    && KEY6_EQ_P(n->key6, key6)
+#if KEY_ARITY > 6
+		    && KEY7_EQ_P(n->key7, key7)
+#if KEY_ARITY > 7
+		    && KEY8_EQ_P(n->key8, key8)
+#if KEY_ARITY > 8
+		    && KEY9_EQ_P(n->key9, key9)
+#endif
+#endif
+#endif
+#endif
 #endif
 #endif
 #endif
@@ -479,7 +713,7 @@ VALTYPE KEYSYM(_stp_map_get) (MAP map, ALLKEYSD(key))
 	return NULLRET;
 }
 
-int KEYSYM(_stp_map_del) (MAP map, ALLKEYSD(key))
+static int KEYSYM(_stp_map_del) (MAP map, ALLKEYSD(key))
 {
 	unsigned int hv;
 	struct hlist_head *head;
@@ -503,6 +737,18 @@ int KEYSYM(_stp_map_del) (MAP map, ALLKEYSD(key))
 		    && KEY4_EQ_P(n->key4, key4)
 #if KEY_ARITY > 4
 		    && KEY5_EQ_P(n->key5, key5)
+#if KEY_ARITY > 5
+		    && KEY6_EQ_P(n->key6, key6)
+#if KEY_ARITY > 6
+		    && KEY7_EQ_P(n->key7, key7)
+#if KEY_ARITY > 7
+		    && KEY8_EQ_P(n->key8, key8)
+#if KEY_ARITY > 8
+		    && KEY9_EQ_P(n->key9, key9)
+#endif
+#endif
+#endif
+#endif
 #endif
 #endif
 #endif
@@ -516,7 +762,7 @@ int KEYSYM(_stp_map_del) (MAP map, ALLKEYSD(key))
 	return 0;
 }
 
-int KEYSYM(_stp_map_exists) (MAP map, ALLKEYSD(key))
+static int KEYSYM(_stp_map_exists) (MAP map, ALLKEYSD(key))
 {
 	unsigned int hv;
 	struct hlist_head *head;
@@ -540,6 +786,18 @@ int KEYSYM(_stp_map_exists) (MAP map, ALLKEYSD(key))
 		    && KEY4_EQ_P(n->key4, key4)
 #if KEY_ARITY > 4
 		    && KEY5_EQ_P(n->key5, key5)
+#if KEY_ARITY > 5
+		    && KEY6_EQ_P(n->key6, key6)
+#if KEY_ARITY > 6
+		    && KEY7_EQ_P(n->key7, key7)
+#if KEY_ARITY > 7
+		    && KEY8_EQ_P(n->key8, key8)
+#if KEY_ARITY > 8
+		    && KEY9_EQ_P(n->key9, key9)
+#endif
+#endif
+#endif
+#endif
 #endif
 #endif
 #endif
@@ -587,6 +845,34 @@ int KEYSYM(_stp_map_exists) (MAP map, ALLKEYSD(key))
 #undef KEY5STOR
 #undef KEY5CPY
 
+#undef KEY6NAME
+#undef KEY6N
+#undef KEY6TYPE
+#undef KEY6_TYPE
+#undef KEY6STOR
+#undef KEY6CPY
+
+#undef KEY7NAME
+#undef KEY7N
+#undef KEY7TYPE
+#undef KEY7_TYPE
+#undef KEY7STOR
+#undef KEY7CPY
+
+#undef KEY8NAME
+#undef KEY8N
+#undef KEY8TYPE
+#undef KEY8_TYPE
+#undef KEY8STOR
+#undef KEY8CPY
+
+#undef KEY9NAME
+#undef KEY9N
+#undef KEY9TYPE
+#undef KEY9_TYPE
+#undef KEY9STOR
+#undef KEY9CPY
+
 #undef KEY_ARITY
 #undef ALLKEYS
 #undef ALLKEYSD
