@@ -19,7 +19,7 @@
 struct stap_perf_probe {
 	struct perf_event_attr attr;
 	perf_overflow_handler_t callback;
-	struct stap_probe probe;
+	struct stap_probe * const probe;
 
 	/* per-cpu data. allocated with _stp_alloc_percpu() */
 	struct perf_event **events;
