@@ -21,6 +21,11 @@
 #define STP_MAX_PROCFS_FILES 16
 #endif
 
+#ifndef STAPCONF_KERN_PATH_PARENT
+#define kern_path_parent(name, nameidata) \
+	path_lookup(name, LOOKUP_PARENT, nameidata)
+#endif
+
 static int _stp_num_pde = 0;
 static struct proc_dir_entry *_stp_pde[STP_MAX_PROCFS_FILES];
 static struct proc_dir_entry *_stp_procfs_files[STP_MAX_PROCFS_FILES];
@@ -55,12 +60,13 @@ static void _stp_rmdir_proc_module(void)
 	}
 
 	if (_stp_proc_stap && _stp_proc_stap->subdir == NULL) {
-		/* Important! Do not attempt removal of /proc/systemtap */
-		/* if in use.  This will put the PDE in deleted state */
-		/* pending usage count dropping to 0. During this time, */
-		/* path_lookup() will still find it and allow new */
-		/* modules to use it, even though it will not show up */
-		/* in directory listings. */
+		/* Important! Do not attempt removal of
+		 * /proc/systemtap if in use.  This will put the PDE
+		 * in deleted state pending usage count dropping to
+		 * 0. During this time, kern_path_parent() will still
+		 * find it and allow new modules to use it, even
+		 * though it will not show up in directory
+		 * listings. */
 
  		if (atomic_read(&_stp_proc_stap->count) == 0) {
 			remove_proc_entry("systemtap", NULL);
@@ -85,12 +91,19 @@ static int _stp_mkdir_proc_module(void)
 			goto done;
 		}
 		
-		/* We use path_lookup() because there is no lookup */
-		/* function for procfs we can call directly.  And */
-		/* proc_mkdir() will always succeed, creating multiple */
-		/* directory entries, all with the same name. */
+		/* We use kern_path_parent() because there is no
+		 * lookup function for procfs we can call directly.
+		 * And proc_mkdir() will always succeed, creating
+		 * multiple directory entries, all with the same
+		 * name.
+		 *
+		 * Why "/proc/systemtap/foo"?  kern_path_parent() is
+		 * basically the same thing as calling the old
+		 * path_lookup() with flags set to LOOKUP_PARENT,
+		 * which means to look up the parent of the path,
+		 * which in this case is "/proc/systemtap". */
 
-		if (path_lookup("/proc/systemtap", 0, &nd)) {
+		if (kern_path_parent("/proc/systemtap/foo", &nd)) {
 			/* doesn't exist, so create it */
 			_stp_proc_stap = proc_mkdir ("systemtap", NULL);
 			if (_stp_proc_stap == NULL) {
@@ -109,7 +122,7 @@ static int _stp_mkdir_proc_module(void)
 		}
 		
 		_stp_proc_root = proc_mkdir(THIS_MODULE->name, _stp_proc_stap);
-#ifdef AUTOCONF_PROCFS_OWNER
+#ifdef STAPCONF_PROCFS_OWNER
 		if (_stp_proc_root != NULL)
 			_stp_proc_root->owner = THIS_MODULE;
 #endif
@@ -167,7 +180,7 @@ static int _stp_create_procfs(const char *path, int num,
 				    goto err;
 			    }
 			    _stp_pde[_stp_num_pde++] = last_dir;
-#ifdef AUTOCONF_PROCFS_OWNER
+#ifdef STAPCONF_PROCFS_OWNER
 			    last_dir->owner = THIS_MODULE;
 #endif
 			    last_dir->uid = _stp_uid;
@@ -191,7 +204,7 @@ static int _stp_create_procfs(const char *path, int num,
 		_stp_error("Could not create file \"%s\" in path \"%s\"\n", p, path);
 		goto err;
 	}
-#ifdef AUTOCONF_PROCFS_OWNER
+#ifdef STAPCONF_PROCFS_OWNER
 	de->owner = THIS_MODULE;
 #endif
 	de->uid = _stp_uid;
