@@ -393,26 +393,22 @@ static int _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 
                     case 'p':
                             /* Note that %p takes an int64_t argument. */
-                            len = 2*sizeof(void *) + 2;
-                            flags |= STP_ZEROPAD;
+                            qualifier = 'L';
+                            /* Since stap 1.3, %p == %#x. */
+                            flags |= STP_SPECIAL;
+                            base = 16;
 
+#if STAP_COMPAT_VERSION < STAP_VERSION(1,3)
+                            /* Before 1.3, %p was an odd child; see below. */
+                            qualifier = 'P';
                             if (field_width == -1)
-                              field_width = len;
+                                    field_width = 2 + 2*sizeof(void*);
+                            precision = field_width - 2;
+                            if (!(flags & STP_LEFT))
+                                    precision = min_t(int, precision, 2*sizeof(void*));
+#endif
 
-                            if (!(flags & STP_LEFT)) {
-                              while (len < field_width) {
-                                field_width--;
-                                num_bytes++;
-                              }
-                            }
-
-                            //account for "0x"
-                            num_bytes+=2;
-                            field_width-=2;
-
-                            num_bytes += number_size((unsigned long) va_arg(args_copy, int64_t),
-                                               16, field_width, field_width, flags);
-                            continue;
+                            break;
 
                     case '%':
                             num_bytes++;
@@ -448,6 +444,8 @@ static int _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 
                     if (qualifier == 'L')
                             num = va_arg(args_copy, int64_t);
+                    else if (qualifier == 'P') // fake, just for compat %p
+                            num = (unsigned long) va_arg(args_copy, int64_t);
                     else if (qualifier == 'l') {
                             num = va_arg(args_copy, unsigned long);
                             if (flags & STP_SIGN)
@@ -688,33 +686,28 @@ static int _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 
 		case 'p':
 			/* Note that %p takes an int64_t argument. */
-			len = 2*sizeof(void *) + 2;
-			flags |= STP_ZEROPAD;
+			qualifier = 'L';
+			/* Since stap 1.3, %p == %#x. */
+			flags |= STP_SPECIAL;
+			base = 16;
 
+#if STAP_COMPAT_VERSION < STAP_VERSION(1,3)
+			/* Before 1.3, %p was an odd child:
+			 * - the value is truncated to unsigned long
+			 * - the specified precision is ignored
+			 * - the default field_width is 2+2*sizeof(void*)
+			 * - the effective precision is field_width - 2, except
+			 *   if !STP_LEFT, where it is at most 2*sizeof(void*)
+			 */
+			qualifier = 'P';
 			if (field_width == -1)
-				field_width = len;
+				field_width = 2 + 2*sizeof(void*);
+			precision = field_width - 2;
+			if (!(flags & STP_LEFT))
+				precision = min_t(int, precision, 2*sizeof(void*));
+#endif
 
-			if (!(flags & STP_LEFT)) {
-				while (len < field_width) {
-					field_width--;
-					if (str <= end)
-						*str = ' ';
-					++str;
-				}
-			}
-			if (str <= end) {
-				*str++ = '0';
-				field_width--;
-			}
-			if (str <= end) {
-				*str++ = 'x';
-				field_width--;
-			}
-
-			str = number(str, end,
-				     (unsigned long) va_arg(args, int64_t),
-				     16, field_width, field_width, flags);
-			continue;
+			break;
 
 		case '%':
 			if (str <= end)
@@ -762,6 +755,8 @@ static int _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 
 		if (qualifier == 'L')
 			num = va_arg(args, int64_t);
+		else if (qualifier == 'P') // fake, just for compat %p
+			num = (unsigned long) va_arg(args, int64_t);
 		else if (qualifier == 'l') {
 			num = va_arg(args, unsigned long);
 			if (flags & STP_SIGN)
