@@ -15,8 +15,9 @@
 
 void register_standard_tapsets(systemtap_session& sess);
 std::vector<derived_probe_group*> all_session_groups(systemtap_session& s);
+std::string common_probe_init (derived_probe* p);
 void common_probe_entryfn_prologue (translator_output* o, std::string statestr,
-				    std::string new_pp, bool overload_processing = true);
+				    std::string probe, bool overload_processing = true);
 void common_probe_entryfn_epilogue (translator_output* o, bool overload_processing = true);
 
 void register_tapset_been(systemtap_session& sess);
@@ -51,11 +52,18 @@ struct var_expanding_visitor: public update_visitor
   std::stack<functioncall**> target_symbol_setter_functioncalls;
   std::stack<defined_op*> defined_ops;
   std::set<std::string> valid_ops;
-  std::string *op;
+  const std::string *op;
 
   var_expanding_visitor ();
   void visit_assignment (assignment* e);
+  void visit_pre_crement (pre_crement* e);
+  void visit_post_crement (post_crement* e);
+  void visit_delete_statement (delete_statement* s);
   void visit_defined_op (defined_op* e);
+
+private:
+  bool rewrite_lvalue(const token *tok, const std::string& eop,
+                      expression*& lvalue, expression*& rvalue);
 };
 
 #endif // TAPSETS_H
