@@ -6,7 +6,8 @@ set m5 0
 set m6 0
 set module1 0
 set kernel 0
-set script_exit 0;
+set script_exit 0
+set eof_found 0
 
 spawn stap -d kernel -d systemtap_test_module1 $srcdir/$subdir/backtrace.stp
 #exp_internal 1
@@ -39,8 +40,9 @@ expect {
 		if {$module1 == 1} {incr kernel}
 		# we expect at least one [kernel] frame, maybe more.
 	    }
+	    eof { set eof_found 1; fail "early test shutdown (m1 eof)" }
 	}
-	exp_continue
+	if {! $eof_found} { exp_continue }
     }
     -re {.*--- yyy_func2 ---\r\nthe stack is 0x[a-f0-9]+ [^\r\n]+\r\n} {
 	incr m2
@@ -62,8 +64,9 @@ expect {
 		if {$module1 == 2} {incr kernel}
 		# we expect at least one [kernel] frame, maybe more.
 	    }
+	    eof { set eof_found 1; fail "early test shutdown (m2 eof)" }
 	}
-	exp_continue
+	if {! $eof_found} { exp_continue }
     }
 
     #backtrace from yyy_func3
@@ -91,8 +94,9 @@ expect {
 		if {$module1 == 3} {incr kernel}
 		# we expect at least one [kernel] frame, maybe more.
 	    }
+	    eof { set eof_found 1; fail "early test shutdown (m3 eof)" }
 	}
-	exp_continue
+	if {! $eof_found} { exp_continue }
     }
     -re {.*--- yyy_func3 ---\r\nthe stack is 0x[a-f0-9]+ [^\r\n]+\r\n} {
 	incr m4
@@ -118,8 +122,9 @@ expect {
 		if {$module1 == 4} {incr kernel}
 		# we expect at least one [kernel] frame, maybe more.
 	    }
+	    eof { set eof_found 1; fail "early test shutdown (m4 eof)" }
 	}
-	exp_continue
+	if {! $eof_found} { exp_continue }
     }
 
     #backtrace from yyy_func4
@@ -151,8 +156,9 @@ expect {
 		if {$module1 == 5} {incr kernel}
 		# we expect at least one [kernel] frame, maybe more.
 	    }
+	    eof { set eof_found 1; fail "early test shutdown (m5 eof)" }
 	}
-	exp_continue
+	if {! $eof_found} { exp_continue }
     }
     -re {.*--- yyy_func4 ---\r\nthe stack is 0x[a-f0-9]+ [^\r\n]+\r\n} {
 	incr m6
@@ -182,8 +188,9 @@ expect {
 		if {$module1 == 6} {incr kernel}
 		# we expect at least one [kernel] frame, maybe more.
 	    }
+	    eof { set eof_found 1; fail "early test shutdown (m6 eof)" }
 	}
-	exp_continue
+	if {! $eof_found} { exp_continue }
     }
     eof {
 	# good backtrace.stp called exit().
